@@ -1,7 +1,7 @@
 setGeneric("ScanBamParam",
            function(flag=scanBamFlag(), simpleCigar=FALSE,
                     reverseComplement=FALSE, tag=character(0),
-                    what=character(0), which)
+                    tagFilter=list(), what=character(0), which)
            standardGeneric("ScanBamParam"),
            signature="which")
 
@@ -123,6 +123,12 @@ setGeneric("yieldTabix", function(file, ..., yieldSize=1000000L)
 
 ## pileup
 
+setGeneric("pileup",
+    function(file, index=file, ..., scanBamParam=ScanBamParam(),
+             pileupParam=PileupParam())
+        standardGeneric("pileup"),
+    signature=signature("file"))
+
 setGeneric(".asSpace", function(x) standardGeneric(".asSpace"))
 
 setGeneric("applyPileups", function(files, FUN, ..., param)
@@ -152,5 +158,20 @@ setGeneric("asMates",
 setGeneric("asMates<-",
            function(object, ..., value) standardGeneric("asMates<-"))
 
+setGeneric("qnamePrefixEnd",
+           function(object, ...) standardGeneric("qnamePrefixEnd"))
+
+setGeneric("qnamePrefixEnd<-",
+           function(object, ..., value) standardGeneric("qnamePrefixEnd<-"))
+
+setGeneric("qnameSuffixStart",
+           function(object, ...) standardGeneric("qnameSuffixStart"))
+
+setGeneric("qnameSuffixStart<-",
+           function(object, ..., value) standardGeneric("qnameSuffixStart<-"))
+
 setGeneric("isOpen")
 
+setGeneric("testPairedEndBam", function(file, index=file, ...) 
+           standardGeneric("testPairedEndBam"),
+           signature="file")
