@@ -1,64 +1,99 @@
-setMethod(ScanBamParam, c(which="missing"),
-          function(flag=scanBamFlag(), simpleCigar=FALSE,
-                   reverseComplement=FALSE, tag=character(0),
-                   what=character(0), which)
-{
-    which <- IRangesList()
-    names(which) <- character()
-    new("ScanBamParam", flag=flag, simpleCigar=simpleCigar,
-        reverseComplement=reverseComplement, tag=tag, what=what,
-        which=which)
-})
-
-setMethod(ScanBamParam, c(which="ANY"),
-          function(flag=scanBamFlag(), simpleCigar=FALSE,
-                   reverseComplement=FALSE, tag=character(0),
-                   what=character(0), which)
-{
-    which <- as(which, "RangesList")
-    new("ScanBamParam", flag=flag, simpleCigar=simpleCigar,
-        reverseComplement=reverseComplement, tag=tag, what=what,
-        which=which)
-})
-
+## Native method first.
 setMethod(ScanBamParam, c(which="RangesList"),
           function(flag=scanBamFlag(), simpleCigar=FALSE,
                    reverseComplement=FALSE, tag=character(0),
-                   what=character(0), which)
+                   tagFilter=list(), what=character(0), which)
 {
+    if (is.null(names(which))) {
+        if (length(which) != 0L)
+            stop(wmsg("'which' must have names in \"ScanBamParam\" ",
+                      "method for RangesList objects "))
+        names(which) <- character()
+    }
     new("ScanBamParam", flag=flag, simpleCigar=simpleCigar,
-        reverseComplement=reverseComplement, tag=tag, what=what,
+        reverseComplement=reverseComplement, tag=tag,
+        tagFilter=.normalize_tagFilter(tagFilter), what=what,
         which=which)
 })
 
+setMethod(ScanBamParam, c(which="missing"),
+          function(flag=scanBamFlag(), simpleCigar=FALSE,
+                   reverseComplement=FALSE, tag=character(0),
+                   tagFilter=list(), what=character(0), which)
+{
+    which <- IRangesList()
+    ScanBamParam(flag=flag, simpleCigar=simpleCigar,
+                 reverseComplement=reverseComplement, tag=tag,
+                 tagFilter=tagFilter, what=what, which=which)
+})
+
+## Default method.
+setMethod(ScanBamParam, c(which="ANY"),
+          function(flag=scanBamFlag(), simpleCigar=FALSE,
+                   reverseComplement=FALSE, tag=character(0),
+                   tagFilter=list(), what=character(0), which)
+{
+    which <- as(which, "RangesList")
+    ScanBamParam(flag=flag, simpleCigar=simpleCigar,
+                 reverseComplement=reverseComplement, tag=tag,
+                 tagFilter=tagFilter, what=what, which=which)
+})
+
+## Note that the 2 methods below are not needed. Coercing a RangedData or
+## GRanges to RangesList works and produces exactly what the methods below
+## are doing by hand. So the default ScanBamParam method above just works
+## on a RangedData or GRanges. -- Herv\'e
 setMethod(ScanBamParam, c(which="RangedData"),
           function(flag=scanBamFlag(), simpleCigar=FALSE,
                    reverseComplement=FALSE, tag=character(0),
-                   what=character(0), which)
+                   tagFilter=list(), what=character(0), which)
 {
+    which <- ranges(which)
     ScanBamParam(flag=flag, simpleCigar=simpleCigar,
-                reverseComplement=reverseComplement, tag=tag, what=what,
-                which=ranges(which))
+                 reverseComplement=reverseComplement, tag=tag,
+                 tagFilter=tagFilter, what=what, which=which)
 })
 
 setMethod(ScanBamParam, c(which="GRanges"),
           function(flag=scanBamFlag(), simpleCigar=FALSE,
                    reverseComplement=FALSE, tag=character(0),
-                   what=character(0), which)
+                   tagFilter=list(), what=character(0), which)
 {
+    which <- split(ranges(which), seqnames(which))
     ScanBamParam(flag=flag, simpleCigar=simpleCigar,
-                reverseComplement=reverseComplement, tag=tag, what=what,
-                which=split(ranges(which), seqnames(which)))
+                 reverseComplement=reverseComplement, tag=tag,
+                 tagFilter=tagFilter, what=what, which=which)
 })
 
+## adapted from ?integer
+.has.wholenumbers <- function(x, tol=.Machine$double.eps^0.5)
+    all(abs(x - round(x)) < tol)
+
+.normalize_tagFilter <- function(tagfilter) {
+    tagfilter <- lapply(tagfilter, function(x) {
+        if(is.numeric(x)) {
+            if(!.has.wholenumbers(x)) {
+                msg <- paste0("Filtering tags by floating point values ",
+                              "not supported. Please use integer values.")
+                stop(paste(strwrap(msg, exdent=2), collapse="\n"))
+            }
+            x <- as.integer(x)
+        }
+        x ##unique(x)
+    })
+    tagfilter
+}
+
 setValidity("ScanBamParam", function(object) {
     msg <- NULL
     flag <- bamFlag(object, asInteger=TRUE)
     simpleCigar <- bamSimpleCigar(object)
     reverseComplement <- bamReverseComplement(object)
     tag <- bamTag(object)
+    tagFilter <- bamTagFilter(object)
     what <- bamWhat(object)
     which <- bamWhich(object)
+    ## flag
     if (length(flag) != 2 || typeof(flag) != "integer")
         msg <- c(msg, "'flag' must be integer(2)")
     else {
@@ -67,16 +102,39 @@ setValidity("ScanBamParam", function(object) {
         else if (any(flag < 0 | flag >= 2^16))
             msg <- c(msg, "'flag' values must be >=0, <2048")
     }
+    ## simpleCigar
     if (!((1L == length(simpleCigar)) &&
           !is.na(simpleCigar)))
         msg <- c(msg, "'simpleCigar' must be logical(1), not NA")
+    ## reverseComplement
     if (!((1L == length(reverseComplement)) &&
           !is.na(reverseComplement)))
         msg <- c(msg, "'reverseComplement' must be logical(1)")
+    ## tag
     if (0 != length(tag) && any(2 != nchar(tag)))
         msg <- c(msg, "'tag' must be two letters, e.g,, 'MD'")
+    ## tagFilter
+    if (length(tagFilter) > 0) {
+        tagnms <- names(tagFilter)
+        if(length(tagnms) == 0L || any(2 != nchar(tagnms)))
+            msg <- c(msg, paste0("'tagFilter' elements must be named with ",
+                                 "two characters each"))
+        for(valvec in tagFilter)
+            if(is.character(valvec) && any(!nzchar(valvec)))
+                msg <- c(msg, "character() tagFilter values must be non-empty")
+        elt_typeinfo <- vapply(tagFilter, function(x) {
+            length(x) && (is.character(x) || is.integer(x)) && ##is.atomic(x)
+            !is.null(x) && !anyNA(x)
+        }, logical(1))
+        if(any(!elt_typeinfo))
+            msg <- c(msg, paste0("'tagFilter' must contain only non-NULL, ",
+                                 "non-NA, non-empty character or integer ",
+                                 "values"))
+    }
+    ## what
     if (!all(what %in% scanBamWhat()))
         msg <- c(msg, "'what' must be from 'scanBamWhat()'")
+    ## which
     if (length(which) != 0 &&
         (any(!nzchar(names(which))) || any(is.na(names(which)))))
         msg <- c(msg, "'which' elements must be named (not NA)")
@@ -99,12 +157,14 @@ bamFlag <- function(object, asInteger=FALSE) {
     ## FIXME: make this support input like bamFlag
 {
     slot(object, "flag") <- value
+    validObject(object)
     object
 }
 bamSimpleCigar <- function(object) slot(object, "simpleCigar")
 "bamSimpleCigar<-" <- function(object, value)
 {
     slot(object, "simpleCigar") <- value
+    validObject(object)
     object
 }
 bamReverseComplement <-
@@ -112,12 +172,21 @@ bamReverseComplement <-
 "bamReverseComplement<-" <- function(object, value)
 {
     slot(object, "reverseComplement") <- value
+    validObject(object)
     object
 }
 bamTag <- function(object) slot(object, "tag")
 "bamTag<-" <- function(object, value)
 {
     slot(object, "tag") <- value
+    validObject(object)
+    object
+}
+bamTagFilter <- function(object) slot(object, "tagFilter")
+"bamTagFilter<-" <- function(object, value)
+{
+    slot(object, "tagFilter") <- .normalize_tagFilter(value)
+    validObject(object)
     object
 }
 
@@ -130,6 +199,7 @@ setReplaceMethod("bamWhich", c("ScanBamParam", "RangesList"),
     function(object, value) 
 {
     slot(object, "which") <- value
+    validObject(object)
     object
 })
 
@@ -137,6 +207,7 @@ setReplaceMethod("bamWhich", c("ScanBamParam", "GRanges"),
     function(object, value) 
 {
     bamWhich(object) <- split(ranges(value), seqnames(value))
+    validObject(object)
     object
 })
         
@@ -144,6 +215,7 @@ setReplaceMethod("bamWhich", c("ScanBamParam", "RangedData"),
     function(object, value) 
 {
     bamWhich(object) <- ranges(value)
+    validObject(object)
     object
 })
         
@@ -151,6 +223,7 @@ setReplaceMethod("bamWhich", c("ScanBamParam", "ANY"),
     function(object, value) 
 {
     bamWhich(object) <-  as(value, "RangesList")
+    validObject(object)
     object
 })
 
@@ -158,6 +231,7 @@ bamWhat <- function(object) slot(object, "what")
 "bamWhat<-" <- function(object, value)
 {
     slot(object, "what") <- value
+    validObject(object)
     object
 }
 
@@ -172,7 +246,7 @@ FLAG_BITNAMES <- c(
     "isMateMinusStrand",
     "isFirstMateRead",
     "isSecondMateRead",
-    "isNotPrimaryRead",
+    "isSecondaryAlignment",
     "isNotPassingQualityControls",
     "isDuplicate"
 )
@@ -182,34 +256,38 @@ scanBamFlag <-
              hasUnmappedMate=NA, isMinusStrand=NA,
              isMateMinusStrand=NA, isFirstMateRead=NA,
              isSecondMateRead=NA, # redundant
-             isNotPrimaryRead=NA, isNotPassingQualityControls=NA,
-             isDuplicate=NA, isValidVendorRead=NA)
+             isNotPrimaryRead=NA, isSecondaryAlignment=NA,
+             isNotPassingQualityControls=NA, isDuplicate=NA)
 
     ## NA: keep either 0 or 1 flag; FALSE: keep 0 flag; TRUE: keep 1 flag
 {
-    flag <- IRanges:::makePowersOfTwo(length(FLAG_BITNAMES))
+    flag <- S4Vectors:::makePowersOfTwo(length(FLAG_BITNAMES))
     names(flag) <- FLAG_BITNAMES
     args <- lapply(as.list(match.call())[-1], eval, parent.frame())
     if (any(sapply(args, length) > 1L))
         stop("all arguments must be logical(1)")
     if (length(args) > 0)
     {
-        ## deprecate isValidVendorRead
-        if ("isValidVendorRead" %in% names(args))
+        ## deprecate isNotPrimaryRead
+        if ("isNotPrimaryRead" %in% names(args))
         {
-            .Deprecated("isNotPassingQualityControls",
-                        old="isValidVendorRead")
-            old <- !args[["isValidVendorRead"]]
-            args[["isValidVendorRead"]] <- NULL
-            if (("isNotPassingQualityControls" %in% names(args)) &&
-                !identical(args[["isNotPassingQualityControls"]], old))
+            .Deprecated("isSecondaryAlignment",
+                        old="isNotPrimaryRead")
+            old <- args[["isNotPrimaryRead"]]
+            args[["isNotPrimaryRead"]] <- NULL
+            value_to_use <- if ("isSecondaryAlignment" %in% names(args))
+                args[["isSecondaryAlignment"]]
+            else
+                old
+            if (("isSecondaryAlignment" %in% names(args)) &&
+                !identical(args[["isSecondaryAlignment"]], old))
             {
                 msg <- sprintf("'%s' inconsistent with '%s', using '%s'",
-                    "isValidVendorRead", "isNotPassingQualityControls",
-                     "isNotPassingQualityControls")
+                               "isNotPrimaryRead", "isSecondaryAlignment",
+                               "isSecondaryAlignment")
                 warning(paste(strwrap(msg, exdent=2), collapse="\n"))
             }
-            args[["isNotPassingQualityControls"]] <- old
+            args[["isSecondaryAlignment"]] <- value_to_use
         }
         ## keep0: NA | FALSE --> drop !NA & TRUE
         idx <- names(args[sapply(args, function(x) !is.na(x) && x)])
@@ -240,6 +318,17 @@ setMethod(show, "ScanBamParam",
     cat("bamReverseComplement: ", bamReverseComplement(object), "\n",
         sep="")
     cat("bamTag:", paste(bamTag(object), collapse=", "), "\n")
+    cat("bamTagFilter:\n")
+    tagFilter <- lapply(bamTagFilter(object), function(x) {
+        if(is.character(x)) sQuote(x) else x
+    })
+    tagnms <- names(tagFilter)
+    for(idx in seq_along(tagFilter)) {
+        elt <- paste(tagnms[[idx]],
+                     paste(BiocGenerics:::selectSome(tagFilter[[idx]]),
+                           collapse=", "), sep=" : ")
+        cat(strwrap(elt, exdent=7, indent=2), sep="\n")
+    }
     cat("bamWhich:", sum(elementLengths(bamWhich(object))), "ranges\n")
     what <- paste("bamWhat: ", paste(bamWhat(object), collapse=", "))
     cat(strwrap(what, exdent=2), sep="\n")
@@ -250,24 +339,57 @@ setMethod(show, "ScanBamParam",
 ## Explode the bits of a 'flag' vector into a matrix.
 bamFlagAsBitMatrix <- function(flag, bitnames=FLAG_BITNAMES)
 {
-    bitpos <- match(bitnames, FLAG_BITNAMES)
-    ans <- IRanges:::explodeIntBits(flag, bitpos=bitpos)
+    ## deprecate 'isNotPrimaryRead'
+    oldname <- "isNotPrimaryRead"
+    newname <- "isSecondaryAlignment"
+    if(oldname %in% bitnames && newname %in% bitnames) {
+        msg <- paste0("'%s' is deprecated form of '%s', please specify ",
+                      "only one")
+        msg <- sprintf(msg, oldname, newname)
+        stop(paste(strwrap(msg, exdent=2), collapse="\n"))
+    }
+    if(oldname %in% bitnames) {
+        .Deprecated(newname, old=oldname)
+        normalized <- bitnames
+        normalized[which(normalized == oldname)] <- newname
+        bitpos <- .calcBitPos(normalized)
+    } else {
+        bitpos <- .calcBitPos(bitnames)
+    }
+    ans <- S4Vectors:::explodeIntBits(flag, bitpos=bitpos)
     dimnames(ans) <- list(names(flag), bitnames)
     ans
 }
 
+.calcBitPos <- function(bitnames) {
+    bitpos <- match(bitnames, FLAG_BITNAMES)
+    invalid_bitnames_idx <- which(is.na(bitpos))
+    if (length(invalid_bitnames_idx) != 0L) {
+        in1string <- paste0(bitnames[invalid_bitnames_idx], collapse=", ")
+        stop("invalid bitname(s): ", in1string)
+    }
+    bitpos
+}
+
 ## Performs a logical AND between 2 'flag' vectors.
 bamFlagAND <- function(flag1, flag2)
 {
     bits1 <- bamFlagAsBitMatrix(flag1)
     bits2 <- bamFlagAsBitMatrix(flag2)
-    ans <- IRanges:::implodeIntBits(bits1 & bits2)
+    ans <- S4Vectors:::implodeIntBits(bits1 & bits2)
     names(ans) <- names(flag1)
     ans
 }
 
 bamFlagTest <- function(flag, value)
 {
+    ## deprecate 'isNotPrimaryRead'
+    oldname <- "isNotPrimaryRead"
+    newname <- "isSecondaryAlignment"
+    if(oldname == value) {
+        .Deprecated(newname, old=oldname)
+        value <- newname
+    }
     if (length(value) != 1 || !value %in% FLAG_BITNAMES) {
         msg <- sprintf("'is' must be character(1) in '%s'",
                        paste(FLAG_BITNAMES, collapse="' '"))
