@@ -1,4 +1,4 @@
-// BamFileIterator.h:
+// BamRangeIterator.h:
 // Iterator used when reading ranges from a bam file.
 
 #ifndef BAMRANGEITERATOR_H
@@ -6,14 +6,12 @@
 
 #include "BamIterator.h"
 
-
 class BamRangeIterator : public BamIterator {
 
+    int32_t tid, beg, end;
     bam_iter_t iter;
 
     void iterate_inprogress(bamFile bfile) {
-	if (iter_done)
-	    return;
 	if (NULL == bam) {	// first record 
 	    bam = bam_init1();
 	    if (bam_iter_read(bfile, iter, bam) < 0) {
@@ -22,12 +20,11 @@ class BamRangeIterator : public BamIterator {
 	    }
 	}
 
-	bool done = false;
 	do {
 	    process(bam);
 	    if (bam_iter_read(bfile, iter, bam) < 0)
-		iter_done = done = true;
-	} while (!done);
+		iter_done = true;
+	} while (!iter_done);
         mate_touched_templates();
     }
 
@@ -35,8 +32,13 @@ class BamRangeIterator : public BamIterator {
         int64_t pos = bam_tell(bfile);
         Templates::iterator it;
         // mate 'inprogress' segments for all templates
-        for (it = templates.begin(); it != templates.end(); ++it) 
-            it->second.mate_inprogress_segments(bfile, bindex, complete);
+        for (it = templates.begin(); it != templates.end(); ++it)
+            it->second.mate_inprogress_segments(bfile, bindex, complete,
+                                                qname_prefix_end(),
+                                                qname_suffix_start(),
+                                                tid, beg, end,
+                                                header->target_len,
+                                                it->first);
 
         BamIterator::finalize_inprogress(bfile);
         bam_seek(bfile, pos, SEEK_SET);
@@ -45,9 +47,9 @@ class BamRangeIterator : public BamIterator {
 public:
 
     // constructor / destructor
-    BamRangeIterator(const bam_index_t *bindex,
-                     int tid, int beg, int end) :
-        BamIterator(bindex) 
+    BamRangeIterator(bamFile bfile, const bam_index_t *bindex,
+                     int32_t tid, int32_t beg, int32_t end) :
+        BamIterator(bfile, bindex), tid(tid), beg(beg), end(end)
     {
 	iter = bam_iter_query(bindex, tid, beg, end);
     }
