@@ -8,6 +8,7 @@
 #include "io_sam.h"
 #include "as_bam.h"
 #include "pileupbam.h"
+#include "pileup.h"
 
 #ifdef _WIN32
 #include "samtools/knetfile.h"
@@ -26,11 +27,11 @@ static const R_CallMethodDef callMethods[] = {
     {".bamfile_close", (DL_FUNC) & bamfile_close, 1},
     {".bamfile_isopen", (DL_FUNC) & bamfile_isopen, 1},
     {".bamfile_isincomplete", (DL_FUNC) & bamfile_isincomplete, 1},
-    {".read_bamfile_header", (DL_FUNC) & read_bamfile_header, 1},
-    {".scan_bamfile", (DL_FUNC) & scan_bamfile, 9},
-    {".count_bamfile", (DL_FUNC) & count_bamfile, 4},
-    {".prefilter_bamfile", (DL_FUNC) & prefilter_bamfile, 7},
-    {".filter_bamfile", (DL_FUNC) & filter_bamfile, 6},
+    {".read_bamfile_header", (DL_FUNC) & read_bamfile_header, 2},
+    {".scan_bamfile", (DL_FUNC) & scan_bamfile, 12},
+    {".count_bamfile", (DL_FUNC) & count_bamfile, 5},
+    {".prefilter_bamfile", (DL_FUNC) & prefilter_bamfile, 10},
+    {".filter_bamfile", (DL_FUNC) & filter_bamfile, 7},
     /* as_bam.c */
     {".as_bam", (DL_FUNC) & as_bam, 3},
     /* io_sam.c */
@@ -51,7 +52,7 @@ static const R_CallMethodDef callMethods[] = {
     {".index_bcf", (DL_FUNC) & index_bcf, 1},
     /* fafile.c */
     {".fafile_init", (DL_FUNC) & fafile_init, 0},
-    {".fafile_open", (DL_FUNC) & fafile_open, 1},
+    {".fafile_open", (DL_FUNC) & fafile_open, 2},
     {".fafile_close", (DL_FUNC) & fafile_close, 1},
     {".fafile_isopen", (DL_FUNC) & fafile_isopen, 1},
     {".index_fa", (DL_FUNC) & index_fa, 1},
@@ -64,17 +65,19 @@ static const R_CallMethodDef callMethods[] = {
     {".tabixfile_isopen", (DL_FUNC) & tabixfile_isopen, 1},
     {".index_tabix", (DL_FUNC) & index_tabix, 8},
     {".header_tabix", (DL_FUNC) & header_tabix, 1},
-    {".scan_tabix", (DL_FUNC) & scan_tabix, 5},
+    {".scan_tabix", (DL_FUNC) & scan_tabix, 6},
     {".tabix_as_character", (DL_FUNC) & tabix_as_character, 4},
-    {".tabix_count", (DL_FUNC) & tabix_count, 4},
-    /* pileup */
+    {".tabix_count", (DL_FUNC) & tabix_count, 5},
+    /* pileupbam */
     {".apply_pileups", (DL_FUNC) & apply_pileups, 5},
     /* bambuffer */
     {".bambuffer_init", (DL_FUNC) & bambuffer_init, 0},
     {".bambuffer", (DL_FUNC) & bambuffer, 1},
     {".bambuffer_length", (DL_FUNC) & bambuffer_length, 1},
-    {".bambuffer_parse", (DL_FUNC) & bambuffer_parse, 7},
+    {".bambuffer_parse", (DL_FUNC) & bambuffer_parse, 8},
     {".bambuffer_write", (DL_FUNC) & bambuffer_write, 3},
+    /* pileup */
+    {".c_Pileup", (DL_FUNC) & c_Pileup, 13},
     {NULL, NULL, 0}
 };
 
