@@ -6,7 +6,6 @@
 #include "utilities.h"
 
 #define BAM_PARSE_STATUS_OK 0
-#define CIGAR_BUFFER_OVERFLOW_ERROR 2
 
 static const int BAM_INIT_SIZE = 1048576;
 
@@ -18,15 +17,16 @@ static BAM_DATA _Calloc_BAM_DATA(int blocksize, int cigar_buf_sz)
 {
     BAM_DATA bd = Calloc(1, _BAM_DATA);
     bd->BLOCKSIZE = blocksize;
-    bd->CIGAR_BUF_SZ = cigar_buf_sz;
-    bd->CIGAR_BUF = Calloc(bd->CIGAR_BUF_SZ, char);
+    bd->cigar_buf_sz = cigar_buf_sz;
+    bd->cigar_buf = Calloc(bd->cigar_buf_sz, char);
     return bd;
 }
 
 BAM_DATA
 _init_BAM_DATA(SEXP ext, SEXP space, SEXP flag, SEXP isSimpleCigar,
-	       int reverseComplement, int yieldSize, int obeyQname,
-	       int asMates, void *extra)
+	       SEXP tagFilter, int reverseComplement, int yieldSize,
+               int obeyQname, int asMates, char qnamePrefixEnd, 
+               char qnameSuffixStart, void *extra)
 {
     int nrange = R_NilValue == space ? 1 : LENGTH(VECTOR_ELT(space, 0));
     BAM_DATA bd =
@@ -40,20 +40,31 @@ _init_BAM_DATA(SEXP ext, SEXP space, SEXP flag, SEXP isSimpleCigar,
     bd->keep_flag[0] = INTEGER(flag)[0];
     bd->keep_flag[1] = INTEGER(flag)[1];
     bd->cigar_flag = LOGICAL(isSimpleCigar)[0];
+    bd->tagfilter = _tagFilter_as_C_types(tagFilter);
     bd->reverseComplement = reverseComplement;
     bd->yieldSize = yieldSize;
     bd->obeyQname = obeyQname;
     bd->asMates = asMates;
+    bd->qnamePrefixEnd = qnamePrefixEnd;
+    bd->qnameSuffixStart = qnameSuffixStart;
     bd->extra = extra;
     return bd;
 }
 
 void _Free_BAM_DATA(BAM_DATA bd)
 {
-    Free(bd->CIGAR_BUF);
+    _Free_C_TAGFILTER(bd->tagfilter);
+    Free(bd->cigar_buf);
     Free(bd);
 }
 
+
+static void _grow_BAM_DATA_cigar(BAM_DATA bd)
+{
+    bd->cigar_buf_sz = bd->cigar_buf_sz * 1.6;
+    bd->cigar_buf = Realloc(bd->cigar_buf, bd->cigar_buf_sz, char);
+}
+
 BAM_FILE _bam_file_BAM_DATA(BAM_DATA bd)
 {
     return bd->bfile;
@@ -123,7 +134,7 @@ static const char *_map(khash_t(str) * h, const char *s)
 static int _bamcigar(const uint32_t * cigar, const uint32_t n_cigar, char *buf,
                      int buf_sz)
 {
-    const char lookup[] = { 'M', 'I', 'D', 'N', 'S', 'H', 'P' };
+    const char lookup[] = BAM_CIGAR_STR;
     buf[0] = '\0';
     for (uint32_t i = 0; i < n_cigar; ++i) {
         int n = snprintf(buf, buf_sz, "%u%c", cigar[i] >> 4,
@@ -256,6 +267,10 @@ int _filter1_BAM_DATA(const bam1_t * bam, BAM_DATA bd)
        test = (keep0 & ~flag) | (keep1 & flag) = 0010 | 1101 = 1111
        ~test = 0000 = FALSE
      */
+    
+    /* tagfilter */
+    if(bd->tagfilter != NULL && !_tagfilter(bam, bd->tagfilter, bd->irec))
+        return 0;
 
     uint32_t test = (bd->keep_flag[0] & ~bam->core.flag) |
         (bd->keep_flag[1] & bam->core.flag);
@@ -318,7 +333,7 @@ int _parse1_BAM_DATA(const bam1_t *bam, BAM_DATA bd)
                 NA_INTEGER : bam_cigar2qlen(&bam->core, bam1_cigar(bam));
             break;
         case MAPQ_IDX:
-            if ((bam->core.flag & BAM_FUNMAP) || (255 == bam->core.qual))
+            if ((bam->core.flag & BAM_FUNMAP))
                 sbd->mapq[idx] = NA_INTEGER;
             else sbd->mapq[idx] = bam->core.qual;
             break;
@@ -326,12 +341,10 @@ int _parse1_BAM_DATA(const bam1_t *bam, BAM_DATA bd)
             if (bam->core.flag & BAM_FUNMAP)
                 sbd->cigar[idx] = NULL;
             else {
-                if (_bamcigar(bam1_cigar(bam), bam->core.n_cigar,
-                              bd->CIGAR_BUF, bd->CIGAR_BUF_SZ) < 0) {
-                    bd->parse_status |= CIGAR_BUFFER_OVERFLOW_ERROR;
-                    return -sbd->icnt;
-                }
-                sbd->cigar[idx] = _map(sbd->cigarhash, bd->CIGAR_BUF);
+                while (_bamcigar(bam1_cigar(bam), bam->core.n_cigar,
+                              bd->cigar_buf, bd->cigar_buf_sz) < 0)
+                    _grow_BAM_DATA_cigar(bd);
+                sbd->cigar[idx] = _map(sbd->cigarhash, bd->cigar_buf);
             }
             break;
         case MRNM_IDX:
