@@ -3,18 +3,27 @@
 
 #include "Rdefines.h"
 #include "bamfile.h"
+#include "tagfilter.h"
+
+#ifdef __cplusplus
+extern "C" {
+#endif
+
+typedef char * (*bam_qname_f)(const bam1_t *bam, char qname_prefix, 
+                              char qname_suffix);
 
 typedef struct {
     int BLOCKSIZE;              /* size to grow vectors */
-    char *CIGAR_BUF;            /* string representation of CIGAR */
-    uint32_t CIGAR_BUF_SZ;
+    char *cigar_buf;            /* string representation of CIGAR */
+    uint32_t cigar_buf_sz;
 
     int parse_status;
     BAM_FILE bfile;
     int irec, iparsed, irange, nrange;
     uint32_t keep_flag[2], cigar_flag;
-    int reverseComplement, yieldSize, 
-        obeyQname, asMates;
+    int reverseComplement, yieldSize, obeyQname, asMates;
+    char qnamePrefixEnd, qnameSuffixStart;
+    C_TAGFILTER tagfilter;
 
     void *extra;
 } _BAM_DATA, *BAM_DATA;
@@ -26,8 +35,9 @@ enum {
 };
 
 BAM_DATA _init_BAM_DATA(SEXP ext, SEXP space, SEXP flag, SEXP isSimpleCigar,
-			int reverseComplement, int yieldSize, int obeyQname,
-			int asMates, void *extra);
+			SEXP tagFilter, int reverseComplement, int yieldSize,
+                        int obeyQname, int asMates, char qnamePrefixEnd, 
+                        char qnameSuffixStart, void *extra);
 void _Free_BAM_DATA(BAM_DATA bd);
 BAM_FILE _bam_file_BAM_DATA(BAM_DATA bd);
 int _count1_BAM_DATA(const bam1_t *bam, BAM_DATA bd);
@@ -36,4 +46,8 @@ int _filter1_BAM_DATA(const bam1_t *bam, BAM_DATA bd);
 int _parse1_BAM_DATA(const bam1_t *bam, BAM_DATA bd);
 void _finish1range_BAM_DATA(BAM_DATA  bd);
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif
