@@ -91,7 +91,8 @@ SEXP bambuffer_length(SEXP bufext)
 }
 
 SEXP bambuffer_parse(SEXP ext, SEXP space, SEXP keepFlags, SEXP isSimpleCigar,
-                     SEXP bufext, SEXP reverseComplement, SEXP templateList)
+                     SEXP tagFilter, SEXP bufext, SEXP reverseComplement,
+                     SEXP templateList)
 {
     _check_isbamfile(ext, "bamBuffer, 'parse'");
     _checkparams(space, keepFlags, isSimpleCigar);
@@ -106,8 +107,8 @@ SEXP bambuffer_parse(SEXP ext, SEXP space, SEXP keepFlags, SEXP isSimpleCigar,
                                       BAMFILE(ext)));
     SCAN_BAM_DATA sbd = _init_SCAN_BAM_DATA(result);
     BAM_DATA bd = _init_BAM_DATA(ext, R_NilValue, keepFlags, isSimpleCigar,
-                                 LOGICAL(reverseComplement)[0],
-                                 NA_INTEGER, 0, 0, (void *) sbd);
+                                 tagFilter, LOGICAL(reverseComplement)[0],
+                                 NA_INTEGER, 0, 0, '\0', '\0', (void *) sbd);
     bd->irange = 0;             /* everything parsed to 'irange' 0 */
     BAM_BUFFER buf = BAMBUFFER(bufext);
     _grow_SCAN_BAM_DATA(bd, buf->n);
