@@ -40,6 +40,8 @@ static void _bamfile_close(SEXP ext)
         bam_index_destroy(bfile->index);
     if (NULL != bfile->iter)
         bam_mate_iter_destroy(bfile->iter);
+    if (NULL != bfile->pbuffer)
+        pileup_pbuffer_destroy(bfile->pbuffer);
     bfile->file = NULL;
     bfile->index = NULL;
     bfile->iter = NULL;
@@ -90,6 +92,7 @@ static BAM_FILE _bamfile_open_r(SEXP filename, SEXP indexname, SEXP filemode)
     }
 
     bfile->iter = NULL;
+    bfile->pbuffer = NULL;
     return bfile;
 }
 
@@ -167,15 +170,20 @@ SEXP bamfile_isincomplete(SEXP ext)
 
 /* implementation */
 
-SEXP read_bamfile_header(SEXP ext)
+SEXP read_bamfile_header(SEXP ext, SEXP what)
 {
     _checkext(ext, BAMFILE_TAG, "scanBamHeader");
-    return _read_bam_header(ext);
+    if (!(IS_LOGICAL(what) && (2L == LENGTH(what))))
+        Rf_error("'what' must be logical(2)");
+    if (!LOGICAL(bamfile_isopen(ext))[0])
+        Rf_error("open() BamFile before reading header");
+    return _read_bam_header(ext, what);
 }
 
 SEXP scan_bamfile(SEXP ext, SEXP space, SEXP keepFlags, SEXP isSimpleCigar,
-                  SEXP reverseComplement, SEXP yieldSize, SEXP template_list,
-                  SEXP obeyQname, SEXP asMates)
+                  SEXP tagFilter, SEXP reverseComplement, SEXP yieldSize,
+                  SEXP template_list, SEXP obeyQname, SEXP asMates,
+                  SEXP qnamePrefixEnd, SEXP qnameSuffixStart)
 {
     _checkext(ext, BAMFILE_TAG, "scanBam");
     _checkparams(space, keepFlags, isSimpleCigar);
@@ -189,22 +197,25 @@ SEXP scan_bamfile(SEXP ext, SEXP space, SEXP keepFlags, SEXP isSimpleCigar,
         Rf_error("'asMates' must be logical(1)");
     _bam_check_template_list(template_list);
     return _scan_bam(ext, space, keepFlags, isSimpleCigar,
-                     reverseComplement, yieldSize, template_list, 
-                     obeyQname, asMates);
+                     tagFilter, reverseComplement, yieldSize, template_list, 
+                     obeyQname, asMates, qnamePrefixEnd, qnameSuffixStart);
 }
 
-SEXP count_bamfile(SEXP ext, SEXP space, SEXP keepFlags, SEXP isSimpleCigar)
+SEXP count_bamfile(SEXP ext, SEXP space, SEXP keepFlags, SEXP isSimpleCigar,
+                   SEXP tagFilter)
 {
     _checkext(ext, BAMFILE_TAG, "countBam");
     _checkparams(space, keepFlags, isSimpleCigar);
-    SEXP count = _count_bam(ext, space, keepFlags, isSimpleCigar);
+    SEXP count = _count_bam(ext, space, keepFlags, isSimpleCigar, tagFilter);
     if (R_NilValue == count)
         Rf_error("'countBam' failed");
     return count;
 }
 
-SEXP prefilter_bamfile(SEXP ext, SEXP space, SEXP keepFlags, SEXP isSimpleCigar,
-                       SEXP yieldSize, SEXP obeyQname, SEXP asMates)
+SEXP prefilter_bamfile(SEXP ext, SEXP space, SEXP keepFlags,
+                       SEXP isSimpleCigar, SEXP tagFilter,
+                       SEXP yieldSize, SEXP obeyQname, SEXP asMates,
+                       SEXP qnamePrefixEnd, SEXP qnameSuffixStart)
 {
     _checkext(ext, BAMFILE_TAG, "filterBam");
     _checkparams(space, keepFlags, isSimpleCigar);
@@ -215,15 +226,16 @@ SEXP prefilter_bamfile(SEXP ext, SEXP space, SEXP keepFlags, SEXP isSimpleCigar,
     if (!(IS_LOGICAL(asMates) && (1L == LENGTH(asMates))))
         Rf_error("'asMates' must be logical(1)");
     SEXP result =
-        _prefilter_bam(ext, space, keepFlags, isSimpleCigar, yieldSize,
-                       obeyQname, asMates);
+        _prefilter_bam(ext, space, keepFlags, isSimpleCigar, tagFilter,
+                       yieldSize, obeyQname, asMates, qnamePrefixEnd,
+                       qnameSuffixStart);
     if (R_NilValue == result)
         Rf_error("'filterBam' failed during pre-filtering");
     return result;
 }
 
 SEXP filter_bamfile(SEXP ext, SEXP space, SEXP keepFlags, SEXP isSimpleCigar,
-                    SEXP fout_name, SEXP fout_mode)
+                    SEXP tagFilter, SEXP fout_name, SEXP fout_mode)
 {
     _checkext(ext, BAMFILE_TAG, "filterBam");
     _checkparams(space, keepFlags, isSimpleCigar);
@@ -231,7 +243,7 @@ SEXP filter_bamfile(SEXP ext, SEXP space, SEXP keepFlags, SEXP isSimpleCigar,
         Rf_error("'fout_name' must be character(1)");
     if (!IS_CHARACTER(fout_mode) || 1 != LENGTH(fout_mode))
         Rf_error("'fout_mode' must be character(1)");
-    SEXP result = _filter_bam(ext, space, keepFlags, isSimpleCigar,
+    SEXP result = _filter_bam(ext, space, keepFlags, isSimpleCigar, tagFilter,
                               fout_name, fout_mode);
     if (R_NilValue == result)
         Rf_error("'filterBam' failed");
