@@ -5,6 +5,7 @@
 #include "samtools/sam.h"
 #include "bambuffer.h"
 #include "bam_mate_iter.h"
+#include "pbuffer_wrapper.h"
 
 #ifdef __cplusplus
 extern "C" {
@@ -15,7 +16,8 @@ typedef struct {
     bam_index_t *index;
     uint64_t pos0;
     int irange0;
-    bam_mate_iter_t iter; 
+    bam_mate_iter_t iter;
+    void *pbuffer; /* for buffered pileup */
 } _BAM_FILE, *BAM_FILE;
 
 #define BAMFILE(b) ((BAM_FILE) R_ExternalPtrAddr(b))
@@ -26,16 +28,21 @@ SEXP bamfile_close(SEXP ext);
 SEXP bamfile_isopen(SEXP ext);
 SEXP bamfile_isincomplete(SEXP ext);
 
-SEXP read_bamfile_header(SEXP ext);
+SEXP read_bamfile_header(SEXP ext, SEXP what);
 SEXP scan_bamfile(SEXP ext, SEXP space, SEXP keepFlags,
-                  SEXP simpleCigar, SEXP reverseComplement,
-                  SEXP yieldSize, SEXP tmpl, SEXP obeyQname, SEXP asMates);
-SEXP count_bamfile(SEXP ext, SEXP space, SEXP keepFlags, SEXP isSimpleCigar);
+                  SEXP simpleCigar, SEXP tagFilter, 
+                  SEXP reverseComplement, SEXP yieldSize,
+                  SEXP tmpl, SEXP obeyQname, 
+                  SEXP asMates, SEXP qnamePrefix, SEXP qnameSuffix);
+SEXP count_bamfile(SEXP ext, SEXP space, SEXP keepFlags, SEXP isSimpleCigar,
+                   SEXP tagFilter);
 SEXP prefilter_bamfile(SEXP ext, SEXP space, SEXP keepFlags,
-		       SEXP isSimpleCigar, SEXP yieldSize, SEXP obeyQname,
-                       SEXP asMates);
+		       SEXP isSimpleCigar, SEXP tagFilter, SEXP yieldSize,
+                       SEXP obeyQname, SEXP asMates, SEXP qnamePrefix,
+                       SEXP qnameSuffix);
 SEXP filter_bamfile(SEXP ext, SEXP space, SEXP keepFlags,
-                    SEXP isSimpleCigar, SEXP fout_name, SEXP fout_mode);
+                    SEXP isSimpleCigar, SEXP tagFilter, SEXP fout_name,
+                    SEXP fout_mode);
 
 void _check_isbamfile(SEXP ext, const char *lbl);
 samfile_t *_bam_tryopen(const char *filename, const char *mode, void *aux);
