@@ -7,9 +7,9 @@
 
 static SEXP FAFILE_TAG = NULL;
 
-static faidx_t *_fa_tryopen(const char *fname)
+static faidx_t *_fa_tryopen(const char *fname, const char *iname)
 {
-    return fai_load(fname);
+    return fai_load0(fname, iname);
 }
 
 static void _fa_close(faidx_t * fai)
@@ -41,15 +41,19 @@ SEXP fafile_init()
     return R_NilValue;
 }
 
-SEXP fafile_open(SEXP filename)
+SEXP fafile_open(SEXP filename, SEXP indexname)
 {
     if (!IS_CHARACTER(filename) || 1 != Rf_length(filename))
         Rf_error("'file' must be character(1)");
+    if (!IS_CHARACTER(indexname) || 1 != Rf_length(indexname))
+        Rf_error("'index' must be character(1)");
 
     _FA_FILE *ffile = Calloc(1, _FA_FILE);
-    const char *cfile = translateChar(STRING_ELT(filename, 0));
+    const char
+        *cfile = translateChar(STRING_ELT(filename, 0)),
+        *ifile = translateChar(STRING_ELT(indexname, 0));
 
-    ffile->index = _fa_tryopen(cfile);
+    ffile->index = _fa_tryopen(cfile, ifile);
     if (NULL == ffile->index) {
         Free(ffile);
         Rf_error("'open' index failed");
@@ -113,12 +117,12 @@ static int translate(Chars_holder *seq_data, const int *lkup, int lkup_length)
     char *dest;
     int nbinvalid, i, j, key, val;
 
-    /* seq_data->seq is a const char * so we need to cast it to
+    /* seq_data->ptr is a const char * so we need to cast it to
        char * before we can write to it */
-    dest = (char *) seq_data->seq;
+    dest = (char *) seq_data->ptr;
     nbinvalid = j = 0;
     for (i = 0; i < seq_data->length; i++) {
-        key = (unsigned char) seq_data->seq[i];
+        key = (unsigned char) seq_data->ptr[i];
         if (key >= lkup_length || (val = lkup[key]) == NA_INTEGER) {
             nbinvalid++;
             continue;
@@ -160,7 +164,7 @@ SEXP scan_fa(SEXP ext, SEXP seq, SEXP start, SEXP end, SEXP type, SEXP lkup)
         Chars_holder ans_elt_holder = get_elt_from_XRawList_holder(&ans_holder, i);
         int len = faidx_fetch_seq2(fai, CHAR(STRING_ELT(seq, i)),
                                    startp[i] - 1, endp[i] - 1,
-                                   (char *) ans_elt_holder.seq);
+                                   (char *) ans_elt_holder.ptr);
         if (len == -1)
             Rf_error(" record %d (%s:%d-%d) failed", i + 1,
                      (char *) CHAR(STRING_ELT(seq, i)), startp[i], endp[i]);
