@@ -6,7 +6,7 @@
 #include "bamfile.h"
 #include "encode.h"
 #include "utilities.h"
-#include "IRanges_interface.h"
+#include "S4Vectors_interface.h"
 #include "XVector_interface.h"
 #include "Biostrings_interface.h"
 #include "bam_mate_iter.h"
@@ -17,8 +17,6 @@ void bam_sort_core(int is_by_qname, const char *fn, const char *prefix,
 
 #define SEQUENCE_BUFFER_ALLOCATION_ERROR 1
 
-typedef void (_FINISH1_FUNC) (BAM_DATA);
-
 static const char *TMPL_ELT_NMS[] = {
     "qname", "flag", "rname", "strand", "pos", "qwidth", "mapq", "cigar",
     "mrnm", "mpos", "isize", "seq", "qual", "tag", "groupid", "mate_status"
@@ -134,36 +132,72 @@ SEXP scan_bam_template(SEXP rname, SEXP tag)
 
 /* header */
 
-SEXP _read_bam_header(SEXP ext)
+SEXP _read_bam_header(SEXP ext, SEXP what)
 {
     samfile_t *sfile = BAMFILE(ext)->file;
     bam_header_t *header = sfile->header;
-    int n_elts = header->n_targets;
 
     SEXP ans = PROTECT(NEW_LIST(2));
-
-    /* target length / name */
-    SET_VECTOR_ELT(ans, 0, NEW_INTEGER(n_elts));
-    SEXP tlen = VECTOR_ELT(ans, 0); /* target length */
-    SEXP tnm = PROTECT(NEW_CHARACTER(n_elts));  /* target name */
-    setAttrib(tlen, R_NamesSymbol, tnm);
-    UNPROTECT(1);
-    for (int j = 0; j < n_elts; ++j) {
-        INTEGER(tlen)[j] = header->target_len[j];
-        SET_STRING_ELT(tnm, j, mkChar(header->target_name[j]));
-    }
-
-    /* 'aux' character string */
-    char *txt = (char *) R_alloc(header->l_text + 1, sizeof(char));
-    strncpy(txt, header->text, header->l_text);
-    txt[header->l_text] = '\0';
-    SET_VECTOR_ELT(ans, 1, mkString(txt));
-
-    SEXP nms = PROTECT(NEW_CHARACTER(2));
+    SEXP nms = NEW_CHARACTER(2);
+    setAttrib(ans, R_NamesSymbol, nms);
     SET_STRING_ELT(nms, 0, mkChar("targets"));
     SET_STRING_ELT(nms, 1, mkChar("text"));
-    setAttrib(ans, R_NamesSymbol, nms);
-    UNPROTECT(2);
+
+    if (LOGICAL(what)[0] == TRUE) { /* 'targets' */
+        int n_elts = header->n_targets;
+        SET_VECTOR_ELT(ans, 0, NEW_INTEGER(n_elts));
+        SEXP tlen = VECTOR_ELT(ans, 0);   /* target length */
+        SEXP tnm = NEW_CHARACTER(n_elts); /* target name */
+        setAttrib(tlen, R_NamesSymbol, tnm);
+        for (int j = 0; j < n_elts; ++j) {
+            INTEGER(tlen)[j] = header->target_len[j];
+            SET_STRING_ELT(tnm, j, mkChar(header->target_name[j]));
+        }
+    }
+
+    if (LOGICAL(what)[1] == TRUE) { /* 'text' */
+        int n_text_elts = 0;
+        for (int i = 0; i < header->l_text; ++i)
+            if (header->text[i] == '\n')
+                n_text_elts += 1;
+        SET_VECTOR_ELT(ans, 1, NEW_LIST(n_text_elts));
+        SEXP text = VECTOR_ELT(ans, 1);
+        SEXP tag = NEW_CHARACTER(n_text_elts);
+        setAttrib(text, R_NamesSymbol, tag);
+
+        int start = 0, end;
+        for (int i = 0; i < n_text_elts; ++i) {
+            int n_elts = header->text[start] == '\n' ? 0 : 1;
+            end = start;
+            while (header->text[end] != '\n') {
+                if (header->text[end] == '\t')
+                    ++n_elts;
+                ++end;
+            }
+            if (n_elts == 0) {
+                SET_VECTOR_ELT(text, i, NEW_CHARACTER(0));
+                /* SET_STRING_ELT(tag, i, mkChar("")); */
+                start = end + 1;
+                continue;
+            }
+            SET_VECTOR_ELT(text, i, NEW_CHARACTER(n_elts - 1));
+            SEXP elts = VECTOR_ELT(text, i);
+
+            for (int j = 0; j < n_elts; ++j) {
+                end = start;
+                while (header->text[end] != '\t' && header->text[end] != '\n')
+                    ++end;
+                SEXP elt = mkCharLen(&header->text[start], end - start);
+                if (j == 0)   /* tag */
+                    SET_STRING_ELT(tag, i, elt);
+                else
+                    SET_STRING_ELT(elts, j - 1, elt);
+                start = end + 1;
+            }
+        }
+    }
+
+    UNPROTECT(1);
     return ans;
 }
 
@@ -235,7 +269,7 @@ int _samread_mate(BAM_FILE bfile, BAM_DATA bd, const int yieldSize,
     bam_mates_t *bam_mates = bam_mates_new();
 
     while (samread_mate(bfile->file->x.bam, bfile->index,
-                        &bfile->iter, bam_mates) > 0) {
+                        &bfile->iter, bam_mates, bd) > 0) {
 
         if (NA_INTEGER != yieldSize && yield  >= yieldSize)
             break;
@@ -268,15 +302,16 @@ static int _scan_bam_all(BAM_DATA bd, bam_fetch_f parse1,
     int yield = 0;
 
     bam_seek(bfile->file->x.bam, bfile->pos0, SEEK_SET);
-    if (bd->asMates) 
+    if (bd->asMates) {
         yield = _samread_mate(bfile, bd, yieldSize, parse1_mate);
-    else
+    } else {
         yield = _samread(bfile, bd, yieldSize, parse1);
+    }
 
     /* end-of-file */
     if ((NA_INTEGER == yieldSize) || (yield < yieldSize))
         bfile->pos0 = bam_tell(bfile->file->x.bam);
-    if (NULL != finish1)
+    if ((NULL != finish1) && (bd->iparsed >= 0))
         (*finish1) (bd);
 
     return bd->iparsed;
@@ -306,12 +341,13 @@ static int _scan_bam_fetch(BAM_DATA bd, SEXP space, int *start, int *end,
             bd->irange += 1;
             return -1;
         }
-        if (bd->asMates)
-            bam_fetch_mate(sfile->x.bam, bindex, tid, starti, end[irange],
-                            bd, parse1_mate);
-        else
+        if (bd->asMates) {
+            bam_fetch_mate(sfile->x.bam, bindex, tid, starti, end[irange], 
+                           bd, parse1_mate);
+        } else {
             bam_fetch(sfile->x.bam, bindex, tid, starti, end[irange],
-                       bd, parse1);
+                      bd, parse1);
+        }
 
         if (NULL != finish1)
             (*finish1) (bd);
@@ -325,15 +361,16 @@ static int _scan_bam_fetch(BAM_DATA bd, SEXP space, int *start, int *end,
     return bd->iparsed - initial;
 }
 
-static int _do_scan_bam(BAM_DATA bd, SEXP space, bam_fetch_f parse1,
-                        bam_fetch_mate_f parse1_mate, _FINISH1_FUNC finish1)
+int _do_scan_bam(BAM_DATA bd, SEXP space, bam_fetch_f parse1,
+                 bam_fetch_mate_f parse1_mate, _FINISH1_FUNC finish1)
 {
     int status;
 
     if (R_NilValue == space)
         /* everything */
         status = _scan_bam_all(bd, parse1, parse1_mate, finish1);
-    else {                      /* fetch */
+    else {                      
+        /* fetch */
         BAM_FILE bfile = _bam_file_BAM_DATA(bd);
         if (NULL == bfile->index)
             Rf_error("valid 'index' file required");
@@ -353,9 +390,11 @@ static int _filter_and_parse1(const bam1_t *bam, void *data)
     /* requires 'data' to be BAM_DATA */
     BAM_DATA bd = (BAM_DATA) data; 
     int result = _filter_and_parse1_BAM_DATA(bam, bd);
-    if (result < 0)
+    if (result < 0) {
         /* parse error: e.g., cigar buf overflow */
         _grow_SCAN_BAM_DATA(bd, 0);
+        bd->iparsed = -1;
+    }
     return result;
 }
 
@@ -366,7 +405,7 @@ static int _filter_and_parse1_mate(const bam_mates_t *mates, void *data)
     SCAN_BAM_DATA sbd = (SCAN_BAM_DATA) bd->extra;
     int yield = 0, pass = 0;
 
-    sbd->mates_flag = mates->mated;
+    sbd->mates_flag = mates->mated == MATE_UNKNOWN ? NA_INTEGER : mates->mated;
     sbd->partition_id += 1;
 
     for (int i = 0; i < mates->n; ++i) {
@@ -427,18 +466,31 @@ SEXP _scan_bam_result_init(SEXP template_list, SEXP names, SEXP space,
 }
 
 SEXP _scan_bam(SEXP bfile, SEXP space, SEXP keepFlags, SEXP isSimpleCigar,
-               SEXP reverseComplement, SEXP yieldSize, SEXP template_list,
-               SEXP obeyQname, SEXP asMates)
+               SEXP tagFilter, SEXP reverseComplement, SEXP yieldSize,
+               SEXP template_list, SEXP obeyQname, SEXP asMates,
+               SEXP qnamePrefixEnd, SEXP qnameSuffixStart)
 {
     SEXP names = PROTECT(GET_ATTR(template_list, R_NamesSymbol));
     SEXP result = PROTECT(_scan_bam_result_init(template_list, names, space,
                                                 BAMFILE(bfile)));
     SCAN_BAM_DATA sbd = _init_SCAN_BAM_DATA(result);
+
+    char qname_prefix = '\0';
+    SEXP prefix_elt = STRING_ELT(qnamePrefixEnd, 0);
+    if (prefix_elt != NA_STRING)
+        qname_prefix = CHAR(prefix_elt)[0];
+    char qname_suffix = '\0';
+    SEXP suffix_elt = STRING_ELT(qnameSuffixStart, 0);
+    if (suffix_elt != NA_STRING)
+        qname_suffix = CHAR(suffix_elt)[0];
+
     BAM_DATA bd = _init_BAM_DATA(bfile, space, keepFlags, isSimpleCigar,
+                                 tagFilter,
                                  LOGICAL(reverseComplement)[0],
                                  INTEGER(yieldSize)[0],
                                  LOGICAL(obeyQname)[0], 
-                                 LOGICAL(asMates)[0], (void *) sbd);
+                                 LOGICAL(asMates)[0], 
+                                 qname_prefix, qname_suffix, (void *) sbd);
 
     int status = _do_scan_bam(bd, space, _filter_and_parse1,
                               _filter_and_parse1_mate, _finish1range_BAM_DATA);
@@ -464,12 +516,13 @@ static int _count1(const bam1_t * bam, void *data)
     return _count1_BAM_DATA(bam, (BAM_DATA) data);
 }
 
-SEXP _count_bam(SEXP bfile, SEXP space, SEXP keepFlags, SEXP isSimpleCigar)
+SEXP _count_bam(SEXP bfile, SEXP space, SEXP keepFlags, SEXP isSimpleCigar,
+    SEXP tagFilter)
 {
     SEXP result = PROTECT(NEW_LIST(2));
     BAM_DATA bd =
-        _init_BAM_DATA(bfile, space, keepFlags, isSimpleCigar, 0,
-                       NA_INTEGER, 0, 0, result);
+        _init_BAM_DATA(bfile, space, keepFlags, isSimpleCigar, tagFilter, 0,
+                       NA_INTEGER, 0, 0, '\0', '\0', result);
 
     SET_VECTOR_ELT(result, 0, NEW_INTEGER(bd->nrange));
     SET_VECTOR_ELT(result, 1, NEW_NUMERIC(bd->nrange));
@@ -523,7 +576,7 @@ static int _prefilter1_mate(const bam_mates_t *mates, void *data)
     int parsed;
 
     buf->partition_id += 1;
-    buf->mate_flag = mates->mated;
+    buf->mate_flag = mates->mated == MATE_UNKNOWN ? NA_INTEGER : mates->mated;
     parsed = 0;
     for (int i = 0; i < mates->n; ++i)
         parsed += _prefilter1(mates->bams[i], data);
@@ -536,14 +589,24 @@ static int _prefilter1_mate(const bam_mates_t *mates, void *data)
 
 SEXP
 _prefilter_bam(SEXP bfile, SEXP space, SEXP keepFlags, SEXP isSimpleCigar,
-               SEXP yieldSize, SEXP obeyQname, SEXP asMates)
+               SEXP tagFilter, SEXP yieldSize, SEXP obeyQname, SEXP asMates, 
+               SEXP qnamePrefixEnd, SEXP qnameSuffixStart)
 {
     SEXP ext = PROTECT(bambuffer(INTEGER(yieldSize)[0],
                                  LOGICAL(asMates)[0]));
+    char qname_prefix = '\0';
+    SEXP prefix_elt = STRING_ELT(qnamePrefixEnd, 0);
+    if (prefix_elt != NA_STRING)
+        qname_prefix = CHAR(prefix_elt)[0];
+    char qname_suffix = '\0';
+    SEXP suffix_elt = STRING_ELT(qnameSuffixStart, 0);
+    if (suffix_elt != NA_STRING)
+        qname_suffix = CHAR(suffix_elt)[0];
     BAM_DATA bd = _init_BAM_DATA(bfile, space, keepFlags, isSimpleCigar,
-                                 0, INTEGER(yieldSize)[0],
+                                 tagFilter, 0, INTEGER(yieldSize)[0],
                                  LOGICAL(obeyQname)[0], 
-                                 LOGICAL(asMates)[0], BAMBUFFER(ext));
+                                 LOGICAL(asMates)[0], 
+                                 qname_prefix, qname_suffix, BAMBUFFER(ext));
     int status =
         _do_scan_bam(bd, space, _prefilter1, _prefilter1_mate, NULL);
     if (status < 0) {
@@ -573,12 +636,12 @@ static int _filter1(const bam1_t * bam, void *data)
 
 SEXP
 _filter_bam(SEXP bfile, SEXP space, SEXP keepFlags,
-            SEXP isSimpleCigar, SEXP fout_name, SEXP fout_mode)
+            SEXP isSimpleCigar, SEXP tagFilter, SEXP fout_name, SEXP fout_mode)
 {
     /* open destination */
     BAM_DATA bd =
-        _init_BAM_DATA(bfile, space, keepFlags, isSimpleCigar, 0,
-                       NA_INTEGER, 0, 0, NULL);
+        _init_BAM_DATA(bfile, space, keepFlags, isSimpleCigar, tagFilter, 0,
+                       NA_INTEGER, 0, 0, '\0', '\0', NULL);
     /* FIXME: this just copies the header... */
     bam_header_t *header = BAMFILE(bfile)->file->header;
     samfile_t *f_out = _bam_tryopen(translateChar(STRING_ELT(fout_name, 0)),
