@@ -1,9 +1,20 @@
 #include <Rdefines.h>
+#include <R_ext/RS.h>
 #include "utilities.h"
 #include "IRanges_interface.h"
 #include "XVector_interface.h"
 
-#include <stdlib.h>  /* for malloc() / free() */
+void *_Rs_Realloc_impl(void *p, size_t n, size_t t)
+{
+    /* Realloc(p, 0, *) fails inappropriately */
+    if (n == 0) {
+        Free(p);
+        p = NULL;
+    } else {
+        p = R_chk_realloc((void *) p, (size_t) (n * t));
+    }
+    return p;
+}
 
 SEXP _get_namespace(const char *pkg)
 {
@@ -23,6 +34,15 @@ void _as_strand(SEXP vec)
     UNPROTECT(2);
 }
 
+void _as_nucleotide(SEXP vec)
+{
+    SEXP nmspc = PROTECT(_get_namespace("Rsamtools"));
+    SEXP lvls = PROTECT(eval(findVar(install(".PILEUP_NUCLEOTIDE_LEVELS"),
+                                     nmspc), nmspc));
+    _as_factor_SEXP(vec, lvls);
+    UNPROTECT(2);
+}
+
 SEXP _get_encoding_lookup(const char *from, const char *to)
 {
     SEXP nmspc, fun, f, t, call, ans;
@@ -52,6 +72,11 @@ SEXP _get_lkup(const char *baseclass)
     return lkup;
 }
 
+void _as_seqlevels(SEXP vec, SEXP lvls)
+{
+    _as_factor_SEXP(vec, lvls);
+}
+
 void _as_factor_SEXP(SEXP vec, SEXP lvls)
 {
     SEXP cls = PROTECT(NEW_CHARACTER(1));
@@ -102,7 +127,7 @@ SEXP _as_XStringSet(const char **key, int len, const char *baseclass)
     for (i = 0; i < len; ++i) {
         const char *seq = key[i];
         dest = get_elt_from_XRawList_holder(&holder, i);
-        Ocopy_bytes_to_i1i2_with_lkup(0, dest.length - 1, (char *) dest.seq,
+        Ocopy_bytes_to_i1i2_with_lkup(0, dest.length - 1, (char *) dest.ptr,
                                       dest.length, seq, strlen(seq), lkup0,
                                       lkup_length);
     }
