@@ -1,12 +1,26 @@
 #ifndef _UTILITIES_H_
 #define _UTILITIES_H_
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
+/* robust memory re-allocation */
+
+#define _Rs_Realloc(p, n, t)	(t *) _Rs_Realloc_impl(p, n, sizeof(t))
+
+void *_Rs_Realloc_impl(void *p, size_t n, size_t t);
+
+/* coercion */
+
 SEXP _get_namespace(const char *pkg);
 SEXP _get_encoding_lookup(const char *from, const char *to);
 SEXP _get_lkup(const char *baseclass);
 void _as_factor_SEXP(SEXP vec, SEXP lvls);
 void _as_factor(SEXP vec, const char **lvls, const int n_lvls);
 void _as_strand(SEXP vec);
+void _as_nucleotide(SEXP vec);
+void _as_seqlevels(SEXP vec, SEXP lvls);
 SEXP _as_XStringSet(const char **key, int len, const char *baseclass);
 SEXP _as_PhredQuality(const char **key, int len);
 void _reverse(char *buf, int len);
@@ -45,4 +59,8 @@ SEXP find_mate_within_groups(SEXP group_sizes,
     GETS = eval(S, ENV);                        \
     UNPROTECT(1)
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif                          /* _UTILITIES_H_ */
